@@ -27,11 +27,14 @@
 #  USA.
 #
 
-vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
-                   sigma2=NULL,hinit=NULL,hincr=NULL,hmax=NULL,lseq=NULL,
-                   u=NULL,graph=FALSE,demo=FALSE,wghts=NULL,na.rm=FALSE,
-                   spmin=.3,scorr=c(0,0,0),vwghts=1,vred="Partial",testprop=FALSE) {
-
+vaws3D <- function(y,qlambda=NULL,lkern="Gaussian",skern="Plateau",weighted=TRUE,
+                   sigma2=NULL,mask=NULL,hinit=NULL,hincr=NULL,hmax=NULL,lseq=NULL,
+                   u=NULL,graph=FALSE,demo=FALSE,wghts=NULL,
+                   spmin=.3,h0=c(0,0,0),vwghts=1,vred="Partial",testprop=FALSE,
+                   res=NULL, resscale=NULL, dim=NULL) {
+#
+#  lseq, qlambda, corrfactor adjusted for case lkern="Gaussian",skern="Plateau" only
+#
   #  Auxilary functions
   IQRdiff <- function(y) IQR(diff(y))/1.908
 
@@ -53,7 +56,6 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
     stop("y has to be 3 or 4 dimensional")
   }
   dv <- dim(y)[d+1]
-
   # define vwghts
   if (length(vwghts)>dv) vwghts <- vwghts[1:dv]
   dv0 <- sum(vwghts>0)
@@ -74,7 +76,7 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
                   2)
 
   # define qlambda, lambda
-  if (is.null(qlambda)) qlambda <- switch(skern,.985,.992,.996)
+  if (is.null(qlambda)) qlambda <- switch(skern,.95,.992,.996)
   if (qlambda<.9) warning("Inappropriate value of qlambda")
   if (qlambda<1) {
     lambda <- qchisq(qlambda,dv0) 
@@ -99,7 +101,7 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
   # re-define bandwidth for Gaussian lkern!!!!
   if (lkern==3) {
     # assume  hmax was given in  FWHM  units (Gaussian kernel will be truncated at 4)
-    hmax <- hmax/sqrt(8*log(2))*4
+    hmax <- fwhm2bw(hmax)*4
     hinit <- min(hinit,hmax)
   }
   if (qlambda == 1) hinit <- hmax
@@ -109,35 +111,33 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
   hincr <- hincr^(1/d)
 
   # determine corresponding bandwidth for specified correlation
-  h0 <- rep(0,length(scorr))
-  if (max(scorr)>0) {
-    h0 <- numeric(length(scorr))
-    for (i in 1:length(h0)) h0[i] <- get.bw.gauss(scorr[i],interv=2)
-    if (length(h0)<d) h0 <- rep(h0[1],d)
-    if (demo) cat("Corresponding bandwiths for specified correlation:",h0,"\n")
-  }
+  if(is.null(h0)) h0 <- rep(0,3)
 
   # estimate variance in the gaussian case if necessary  
   if (is.null(sigma2)) {
     sigma2 <- IQRdiff(as.vector(y))^2
-    if (scorr[1]>0) sigma2<-sigma2*Varcor.gauss(h0)*Spatialvar.gauss(h0*c(1,wghts),1e-5,d)
+    if (any(h0>0)) sigma2<-sigma2*Varcor.gauss(h0)*Spatialvar.gauss(h0,1e-5,d)
     if (demo) cat("Estimated variance: ", signif(sigma2,4),"\n")
   }
   if (length(sigma2)==1) sigma2<-array(sigma2,dy[1:3]) 
+  if(is.null(mask)) mask <- array(TRUE,dy[1:3])
+  mask[sigma2>=1e16] <- FALSE
+#  in these points sigma2 probably contains NA's
   # deal with homoskedastic Gaussian case by extending sigma2
   if (length(sigma2)!=n) stop("sigma2 does not have length 1 or same length as y")
+  dim(sigma2) <- dy[1:3]
   lambda <- lambda*2 
   sigma2 <- 1/sigma2 #  taking the invers yields simpler formulaes 
 
   # demo mode should deliver graphics!
   if (demo && !graph) graph <- TRUE
   
-  # Initialize  list for ai, bi and theta
+  # Initialize  list for bi and theta
   if (is.null(wghts)) wghts <- c(1,1,1)
   hinit <- hinit/wghts[1]
   hmax <- hmax/wghts[1]
   wghts <- (wghts[2:3]/wghts[1])
-  tobj <- list(ai=y, bi= rep(1,n))
+  tobj <- list(bi= rep(1,n))
   theta <- y
   steps <- as.integer(log(hmax/hinit)/log(hincr)+1)
 
@@ -146,7 +146,7 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
 # this is optimized for lkern="Gaussian" such that alpha approx 0.04 -- 0.1 and probability of separated points is approx. 1e-4
     lseqexp <- c(rep(1.286,11), 1.21, 1.21, 1.14, 1.14, 1.07, 1.07)# alpha=0.1       prob: .36e-4
     lseqtriangle <- c(1.79, 1.64, 1.57, 1.5, 1.5, 1.43, 1.43, 1.36, 1.36, 1.29, 1.29, 1.21, 1.21, 1.14, 1.14, 1.07, 1.07) # alpha=0.1       prob: .5e-4
-    lseqplateau  <- c(rep(1.25,10),1.1875,1.125,rep(1.0625,3),1)# alpha=0.056       prob: .24e-4
+    lseqplateau  <- c(rep(1.25,10),1.2,1.15,1.1,1.1,1.05,1.05,1)# alpha=0.1       prob: .24e-4
     lseq <- switch(skern,lseqplateau,lseqtriangle,lseqexp)
   }
   if (length(lseq)<steps) lseq <- c(lseq,rep(1,steps-length(lseq)))
@@ -157,7 +157,10 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
   hakt0 <- hinit
   lambda0 <- lambda*lseq[k]
   if (hinit>1) lambda0 <- 1e50 # that removes the stochstic term for the first step
-  
+  scorr <- numeric(3)
+  if(h0[1]>0) scorr[1] <-  get.corr.gauss(h0[1],2)
+  if(h0[2]>0) scorr[2] <-  get.corr.gauss(h0[2],2)
+  if(h0[3]>0) scorr[3] <-  get.corr.gauss(h0[3],2)
   progress <- 0
   step <- 0
   total <- (hincr^(d*ceiling(log(hmax/hinit)/log(hincr)))-1)/(hincr^d-1)
@@ -168,9 +171,9 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
   # run single steps to display intermediate results
   while (hakt<=hmax) {
     dlw <- (2*trunc(hakt/c(1,wghts))+1)[1:d]
-    if (scorr[1]>=0.1) lambda0 <- lambda0 * Spatialvar.gauss(hakt0/0.42445/4*c(1,wghts),h0*c(1,wghts),d) /
-      Spatialvar.gauss(h0*c(1,wghts),1e-5,d) /
-        Spatialvar.gauss(hakt0/0.42445/4*c(1,wghts),1e-5,d)
+#  need bandwidth in voxel for Spaialvar.gauss, h0 is in voxel
+    if (any(h0>0)) lambda0 <- lambda0 * Spatialvar.gauss(bw2fwhm(hakt0)/4/c(1,wghts),h0,d)/
+      Spatialvar.gauss(h0,1e-5,d)/Spatialvar.gauss(bw2fwhm(hakt0)/4/c(1,wghts),1e-5,d)
         # Correction C(h0,hakt) for spatial correlation depends on h^{(k-1)}  all bandwidth-arguments in FWHM 
     hakt0 <- hakt
     theta0 <- theta
@@ -180,6 +183,8 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
     #
     tobj <- .Fortran("chaws2",as.double(y),
                      as.double(sigma2),
+                     as.logical(!mask),
+                     as.logical(weighted),
                      as.integer(n1),
                      as.integer(n2),
                      as.integer(n3),
@@ -189,7 +194,7 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
                      as.double(lambda0),
                      as.double(theta0),
                      bi=as.double(bi0),
-                     ai=as.double(tobj$ai),
+                     thnew=as.double(y),
                      as.integer(lkern),
                      as.integer(skern),
                      as.double(spmin),
@@ -199,16 +204,15 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
                      as.double(vwghts),
                      double(dv),#swjy
                      double(dv0),#thi
-		     as.logical(na.rm),#narm
-                     PACKAGE="fmri",DUP=TRUE)[c("bi","ai","hakt")]
+                     PACKAGE="fmri",DUP=TRUE)[c("bi","thnew","hakt")]
     gc()
-    dim(tobj$ai) <- dy
-    gc()
-    theta <- array(tobj$ai/tobj$bi,dy) 
+    theta <- array(tobj$thnew,dy) 
     dim(tobj$bi) <- dy[-4]
     if(testprop) {
       pobj <- .Fortran("chaws2",as.double(y),
                        as.double(sigma2),
+                       as.logical(!mask),
+                       as.logical(weighted),
                        as.integer(n1),
                        as.integer(n2),
                        as.integer(n3),
@@ -218,7 +222,7 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
                        as.double(1e50),
                        as.double(theta0),
                        bi=as.double(bi0),
-                       ai=double(n1*n2*n3*dv),
+                       thnew=double(n1*n2*n3*dv),
                        as.integer(lkern),
 		       as.integer(skern),
 	               as.double(spmin),
@@ -228,9 +232,8 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
 		       as.double(vwghts),
 		       double(dv),#swjy
 		       double(dv0),#thi
-                       as.logical(na.rm),#narm
-		       PACKAGE="fmri",DUP=TRUE)[c("bi","ai")]
-      ptheta <- array(pobj$ai/pobj$bi,dy) 
+		       PACKAGE="fmri",DUP=TRUE)[c("bi","thnew")]
+      ptheta <- array(pobj$thnew,dy) 
       rm(pobj) 
       gc()
       propagation <- c(propagation,sum(abs(theta-ptheta))/sum(abs(ptheta-u)))
@@ -248,7 +251,7 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
       title(paste("Sum of weights: min=",signif(min(tobj$bi),3)," mean=",signif(mean(tobj$bi),3)," max=",signif(max(tobj$bi),3)))
     }
     if (!is.null(u)) {
-      cat("bandwidth: ",signif(hakt*switch(lkern,1,1,sqrt(8*log(2))/4),3),"eta==1",sum(tobj$eta==1),"   MSE: ",
+      cat("bandwidth: ",signif(hakt*switch(lkern,hakt,hakt,bw2fwhm(hakt)/4),3),"eta==1",sum(tobj$eta==1),"   MSE: ",
           signif(mean((theta-u)^2),3),"   MAE: ",signif(mean(abs(theta-u)),3)," mean(bi)=",signif(mean(tobj$bi),3),"\n")
       mae <- c(mae,signif(mean(abs(theta-u)),3))
     } else if (total !=0) {
@@ -258,30 +261,35 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
     }
     if (demo) readline("Press return")
     hakt <- hakt*hincr
-    x <- 1.25^(k-1)
-    scorrfactor <- x/(3^d*prod(scorr)*prod(h0)+x)
+#  adjust lambda for the high intrinsic correlation between  neighboring estimates 
+    c1 <- (prod(h0+1))^(1/3)
+    c1 <- 2.7214286 - 3.9476190*c1 + 1.6928571*c1*c1 - 0.1666667*c1*c1*c1
+    x <- (prod(1.25^(k-1)/c(1,wghts)))^(1/3)
+    scorrfactor <- (c1+x)/(c1*prod(h0+1)+x)
     lambda0 <- lambda*lseq[k]*scorrfactor
     k <- k+1
     gc()
   }
 
   #   Now compute variance of theta and variance reduction factor (with respect to the spatially uncorrelated situation   
-  g <- trunc(h0/c(1,wghts)/2.3548*4)
+if(is.null(res)){
+  g <- trunc(fwhm2bw(h0)*4)
 
-  #  use g <- trunc(h0/c(1,wghts)/2.3548*3)+1 if it takes to long
   cat("estimate variances .")
-  if(h0[1]>0) gw1 <- dnorm(-(g[1]):g[1],0,h0[1]/2.3548)/dnorm(0,0,h0[1]/2.3548) else gw1 <- 1
-  if(h0[2]>0) gw2 <- dnorm(-(g[2]):g[2],0,h0[2]/2.3548/wghts[1])/dnorm(0,0,h0[2]/2.3548/wghts[1]) else gw2 <- 1
-  if(h0[3]>0) gw3 <- dnorm(-(g[3]):g[3],0,h0[3]/2.3548/wghts[2])/dnorm(0,0,h0[3]/2.3548/wghts[2]) else gw3 <- 1
+  if(h0[1]>0) gw1 <- dnorm(-(g[1]):g[1],0,fwhm2bw(h0[1]))/dnorm(0,0,fwhm2bw(h0[1])) else gw1 <- 1
+  if(h0[2]>0) gw2 <- dnorm(-(g[2]):g[2],0,fwhm2bw(h0[2]))/dnorm(0,0,fwhm2bw(h0[2])) else gw2 <- 1
+  if(h0[3]>0) gw3 <- dnorm(-(g[3]):g[3],0,fwhm2bw(h0[3]))/dnorm(0,0,fwhm2bw(h0[3])) else gw3 <- 1
   gwght <- outer(gw1,outer(gw2,gw3,"*"),"*")
   gwght <- gwght/sum(gwght)
   dgw <- dim(gwght)
-  if (scorr[1]>=0.1) lambda0 <- lambda0 * Spatialvar.gauss(hakt0/hincr/0.42445/4*c(1,wghts),h0*c(1,wghts),d) /
-    Spatialvar.gauss(h0*c(1,wghts),1e-5,d) /
-      Spatialvar.gauss(hakt0/hincr/0.42445/4*c(1,wghts),1e-5,d)
+  if (any(h0)>0) lambda0 <- lambda0 * Spatialvar.gauss(bw2fwhm(hakt0/hincr)/4/c(1,wghts),h0,d) /
+    Spatialvar.gauss(h0,1e-5,d) /
+      Spatialvar.gauss(bw2fwhm(hakt0/hincr)/4/c(1,wghts),1e-5,d)
   if(vred=="Full"){
     z <- .Fortran("chawsvr",
                   as.double(sigma2),
+                  as.logical(!mask),
+                  as.logical(weighted),
                   as.integer(n1),
                   as.integer(n2),
                   as.integer(n3),
@@ -303,12 +311,13 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
                   as.double(wghts),
                   as.double(vwghts),
                   double(dv0),#thi
-                  as.logical(na.rm),#narm
                   PACKAGE="fmri",DUP=FALSE)[c("vred","var")]
     dim(z$vred) <- dim(z$var) <- dim(sigma2)
   } else {
     z1 <- .Fortran("chawsvr1",
                    as.double(sigma2),
+                   as.logical(!mask),
+                   as.logical(weighted),
                    as.integer(n1),
                    as.integer(n2),
                    as.integer(n3),
@@ -328,7 +337,6 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
                    as.double(wghts),
                    as.double(vwghts),
                    double(dv0),#thi
-                   as.logical(na.rm),#narm
                    PACKAGE="fmri",DUP=FALSE)[c("bi0","bi2","Qh","Qh0")]
     bi2 <- array(z1$bi2,dim(sigma2))
     bi0 <- array(z1$bi0,dim(sigma2))
@@ -366,17 +374,148 @@ vaws3D <- function(y,qlambda=NULL,lkern="Triangle",skern="Plateau",
   if(vred=="Full"){
     vartheta <- z$var/tobj$bi^2/qg
     vred <- z$vred/tobj$bi^2/ng^2
-    vred0 <- vred
   } else {
-    vartheta <- Qhg/Qh0/qg*bi2/tobj$bi^2
-    vred <- Qhg/Qh0/ng^2*Qh/tobj$bi^2
-    vred0 <- Qhg/Qh0/ng^2*Qh/bi0^2
+#    vartheta <- Qhg/Qh0/qg*bi2/tobj$bi^2
+    vartheta <- Qhg/Qh0/qg*bi2/bi0^2
+#    vred <- Qhg/Qh0/ng^2*Qh/tobj$bi^2
+    vred <- Qhg/Qh0/ng^2*Qh/bi0^2
   }
- 
-  #   vred accounts for variance reduction with respect to uncorrelated (\check{sigma}^2) data
-  z <- list(theta=theta,ni=tobj$bi,var=vartheta,vred=vred,vred0=vred0,y=y,
-            hmax=tobj$hakt*switch(lkern,1,1,sqrt(8*log(2))/4),mae=mae,lseq=c(0,lseq[-steps]),call=args,ng=ng,qg=qg,alpha=propagation)
+  z <- list(theta=theta,ni=tobj$bi,var=vartheta,vred=vred,vred0=median(vred[mask]),y=y,
+            hmax=tobj$hakt*switch(lkern,1,1,bw2fwhm(1/4)),mae=mae,lseq=c(0,lseq[-steps]),call=args,ng=ng,qg=qg,alpha=propagation)
 # Bandwidth in FWHM in case of lkern="Gaussian"
+} else {
+  residuals <- readBin(res,"integer",prod(dim),2)
+  cat("\nVaws3D: first variance estimation","\n")
+  vartheta0 <- .Fortran("ivar",as.integer(residuals),
+                           as.double(resscale),
+                           as.logical(mask),
+                           as.integer(n1),
+                           as.integer(n2),
+                           as.integer(n3),
+                           as.integer(dim[4]),
+                           var = double(n1*n2*n3),
+                           PACKAGE="fmri",DUP=FALSE)$var
+  cat("Vaws3D: smooth the residuals","\n")
+  residuals <- .Fortran("ihaws2",as.integer(residuals),
+                     as.double(sigma2),
+                     as.logical(!mask),
+                     as.logical(weighted),
+                     as.integer(n1),
+                     as.integer(n2),
+                     as.integer(n3),
+                     as.integer(dim[4]),
+                     as.integer(dv0),
+                     hakt=as.double(tobj$hakt),
+                     as.double(lambda0),
+                     as.double(theta0),
+                     bi=as.double(bi0),
+                     resnew=as.integer(residuals),
+                     as.integer(lkern),
+                     as.integer(skern),
+                     as.double(spmin),
+                     as.double(spmax),
+                     double(prod(dlw)),
+                     as.double(wghts),
+                     as.double(vwghts),
+                     double(dim[4]),#swjy
+                     double(dv0),#thi
+                     PACKAGE="fmri",DUP=FALSE)$resnew
+  dim(residuals) <- dim
+  gc()
+#   get variances and correlations
+  cat("Vaws3D: estimate correlations","\n")
+  lags <- c(5,5,3)
+  scorr <- .Fortran("imcorr",as.integer(residuals),
+                     as.logical(mask),
+                     as.integer(dy[1]),
+                     as.integer(dy[2]),
+                     as.integer(dy[3]),
+                     as.integer(dy[4]),
+                     scorr=double(prod(lags)),
+                     as.integer(lags[1]),
+                     as.integer(lags[2]),
+                     as.integer(lags[3]),
+                     PACKAGE="fmri",DUP=FALSE)$scorr
+  dim(scorr) <- lags                     
+  cat("Vaws3D: final variance estimation","\n")
+  vartheta <- .Fortran("ivar",as.integer(residuals),
+                           as.double(resscale),
+                           as.logical(mask),
+                           as.integer(n1),
+                           as.integer(n2),
+                           as.integer(n3),
+                           as.integer(dim[4]),
+                           var = double(n1*n2*n3),
+                           PACKAGE="fmri",DUP=FALSE)$var
+  vred <- array(vartheta/vartheta0,c(n1,n2,n3))
+  vartheta <- vred/sigma2  #  sigma2 contains inverse variances
+  z <- list(theta=theta,ni=tobj$bi,var=vartheta,vred=vred,vred0=median(vred[mask]),y=y,
+            hmax=tobj$hakt*switch(lkern,1,1,bw2fwhm(1/4)),mae=mae,
+            lseq=c(0,lseq[-steps]),call=args,alpha=propagation,scorr=scorr)
+}
+  #   vred accounts for variance reduction with respect to uncorrelated (\check{sigma}^2) data
   class(z) <- "aws.gaussian"
   invisible(z)
 }
+
+smooth3D <- function(y,lkern="Gaussian",weighted=FALSE,sigma2=NULL,mask=NULL,hmax=NULL,
+                     wghts=NULL) {
+  #
+  # first check arguments and initialize
+  #
+  args <- match.call()
+
+  # test dimension of data (vector of 3D) and define dimension related stuff
+  d <- 3
+  dy <- dim(y)
+  n1 <- dy[1]
+  n2 <- dy[2]
+  n3 <- dy[3]
+  n <- n1*n2*n3
+  if (length(dy)==d) {
+    dim(y) <- dy <- c(dy,1)
+  } else if (length(dy)!=d+1) {
+    stop("y has to be 3 or 4 dimensional")
+  }
+  dv <- dim(y)[d+1]
+    if(is.null(sigma2)) {
+       weighted <- FALSE
+    } else {
+      if(length(sigma2)!=n) weighted <- FALSE
+      sigma2 <- 1/sigma2
+    }
+  if (is.null(hmax)) hmax <- 5    # uses a maximum of about 520 points
+
+  # re-define bandwidth for Gaussian lkern!!!!
+  lkern <- switch(lkern,
+                  Triangle=2,
+                  Plateau=1,
+                  Gaussian=3,
+                  1)
+  if (lkern==3) {
+    # assume  hmax was given in  FWHM  units (Gaussian kernel will be truncated at 4)
+    hmax <- fwhm2bw(hmax)*4
+  }
+  if (is.null(wghts)) wghts <- c(1,1,1)
+  if(is.null(mask)) mask <- array(TRUE,dy[1:3])
+  hmax <- hmax/wghts[1]
+  wghts <- (wghts[2:3]/wghts[1])
+    dlw <- (2*trunc(hmax/c(1,wghts))+1)[1:d]
+    ysmooth <- .Fortran("smooth3d",
+                     as.double(y),
+                     as.double(sigma2),
+                     as.logical(!mask),
+                     as.logical(weighted),
+                     as.integer(n1),
+                     as.integer(n2),
+                     as.integer(n3),
+                     as.integer(dv),
+                     hakt=as.double(hmax),
+                     thnew=as.double(y),
+                     as.integer(lkern),
+                     double(prod(dlw)),
+                     as.double(wghts),
+                     double(dv),#swjy
+                     PACKAGE="fmri",DUP=TRUE)$thnew
+array(ysmooth,dy)
+}
